#include <stdio.h>

#include "sslauth.h"

#define _pam_overwrite(x)        \
do {                             \
     register char *__xx__;      \
     if ((__xx__=(x)))           \
          while (*__xx__)        \
               *__xx__++ = '\0'; \
} while (0)

#define _pam_drop_reply(/* struct pam_response * */ reply, /* int */ replies) \
do {                                              \
    int reply_i;                                  \
                                                  \
    for (reply_i=0; reply_i<replies; ++reply_i) { \
	if (reply[reply_i].resp) {                \
	    _pam_overwrite(reply[reply_i].resp);  \
	    free(reply[reply_i].resp);            \
	}                                         \
    }                                             \
    if (reply)                                    \
	free(reply);                              \
} while (0)

static int return_null_password(char *buf) {
    if (buf) buf[0]=0;
    return 0;
}

int get_key_password(char *buf, int size, int rwflag, void *userdata) {
    pam_handle_t *pamh;
    char *certname;
    char prompt[MAX_FILENAME_SIZE+12];

    int retval;
    struct pam_conv *conv;
    struct pam_message msg;
    const struct pam_message *pmsg;
    struct pam_response *prsp;
    
    pamh=(pam_handle_t*)(((struct _pam_userdata*)(userdata))->pamh);
    certname=(char*)(((struct _pam_userdata*)(userdata))->certname);
    
    sprintf(prompt,"Password(%s.key): ",certname);
    
    retval = pam_get_item(pamh, PAM_CONV, (const void **) &conv);
    if (retval != PAM_SUCCESS) {
	_pam_log(LOG_ERR, "Couldn't obtain coversation function [%s]", pam_strerror(pamh, retval));
	return return_null_password(buf);
    }

    msg.msg = prompt;
    msg.msg_style = PAM_PROMPT_ECHO_OFF;
    pmsg=&msg;
    retval = conv->conv(1,&pmsg,&prsp,conv->appdata_ptr);
    if ((!prsp)||(retval != PAM_SUCCESS)) {
	_pam_log(LOG_ERR, "Conversation failed!");
	if (prsp) _pam_drop_reply(prsp, 1);
	return return_null_password(buf);
    } else {
	if (prsp->resp)
	    strncpy(buf,prsp->resp,size-1);buf[size-1]=0;
	_pam_drop_reply(prsp, 1);
    }
    return strlen(buf);
}
